@@ -22,7 +22,6 @@ import matplotlib.pyplot as plt
 import sectionproperties.pre.pre as pre
 import sectionproperties.pre.bisect_section as bisect
 import sectionproperties.post.post as post
-import sectionproperties.pre.rhino as rhino_importer
 
 
 class Geometry:
@@ -262,6 +261,16 @@ class Geometry:
                 If true, all non parallel surfaces are filtered out.
                 Default is False.
         """
+        try:
+            import sectionproperties.pre.rhino as rhino_importer  # type: ignore
+        except ImportError as e:
+            print(e)
+            print(
+                "There is something wrong with your rhino library installation. "
+                "Please report this error at https://github.com/robbievanleeuwen/section-properties/issues"
+            )
+            return
+
         geom = None
         list_poly = rhino_importer.load_3dm(filepath, **kwargs)
         if len(list_poly) == 1:
@@ -313,6 +322,15 @@ class Geometry:
                 If true, all non parallel surfaces are filtered out.
                 Default is False.
         """
+        try:
+            import sectionproperties.pre.rhino as rhino_importer  # type: ignore
+        except ImportError as e:
+            print(e)
+            print(
+                "There is something wrong with your rhino library installation. "
+                "Please report this error at https://github.com/robbievanleeuwen/section-properties/issues"
+            )
+            return
         return cls(geom=rhino_importer.load_brep_encoding(r3dm_brep, **kwargs)[0])
 
     def create_facets_and_control_points(self):
@@ -339,12 +357,14 @@ class Geometry:
         """
         self.create_facets_and_control_points()
 
-    def create_mesh(self, mesh_sizes: Union[float, List[float]]):
+    def create_mesh(self, mesh_sizes: Union[float, List[float]], coarse: bool = False):
         """Creates a quadratic triangular mesh from the Geometry object.
 
         :param mesh_sizes: A float describing the maximum mesh element area to be used
             within the Geometry-object finite-element mesh.
         :type mesh_sizes: Union[float, List[float]]
+        :param bool coarse: If set to True, will create a coarse mesh (no area or
+            quality constraints)
 
         :return: Geometry-object with mesh data stored in .mesh attribute. Returned
             Geometry-object is self, not a new instance.
@@ -374,7 +394,7 @@ class Geometry:
                 f"a float, or a list of float with length of 1, not {mesh_sizes}."
             )
         self.mesh = pre.create_mesh(
-            self.points, self.facets, self.holes, self.control_points, mesh_size
+            self.points, self.facets, self.holes, self.control_points, mesh_size, coarse
         )
         return self
 
@@ -1038,25 +1058,56 @@ class Geometry:
         """
         Perform difference on Geometry objects with the - operator
         """
-        material = self.material or other.material
-        try:
-            new_polygon = filter_non_polygons(self.geom - other.geom)
-            if isinstance(new_polygon, MultiPolygon):
-                return CompoundGeometry(
-                    [Geometry(polygon, material) for polygon in new_polygon.geoms]
+        material = pre.DEFAULT_MATERIAL
+        if isinstance(self, CompoundGeometry):
+            subs_geom_acc = []
+            for geom in self.geoms:
+                new_geom = geom - other
+                subs_geom_acc.append(new_geom)
+            return CompoundGeometry(subs_geom_acc)
+        else:
+            material = self.material or pre.DEFAULT_MATERIAL
+            try:
+                new_polygon = filter_non_polygons(self.geom - other.geom)
+                if isinstance(new_polygon, GeometryCollection):  # Non-polygon results
+                    return None
+                elif isinstance(new_polygon, MultiPolygon):
+                    return CompoundGeometry(
+                        [Geometry(polygon, material) for polygon in new_polygon.geoms]
+                    )
+                elif isinstance(new_polygon, Polygon):
+                    if self.assigned_control_point and new_polygon.contains(
+                        self.assigned_control_point
+                    ):
+                        return Geometry(
+                            new_polygon, material, self.assigned_control_point
+                        )
+                    else:
+                        return Geometry(new_polygon, material)
+            except:
+                raise ValueError(
+                    f"Cannot perform 'difference' on these two objects: {self} - {other}"
                 )
-            if isinstance(new_polygon, GeometryCollection):
-                return None
-            # Check to see if assigned_control_point is still valid
-            if self.assigned_control_point and new_polygon.contains(
-                self.assigned_control_point
-            ):
-                return Geometry(new_polygon, material, self.control_points[0])
-            return Geometry(new_polygon, material)
-        except:
-            raise ValueError(
-                f"Cannot perform 'difference' on these two objects: {self} - {other}"
-            )
+
+        # material = self.material or other.material
+        # try:
+        #     new_polygon = filter_non_polygons(self.geom - other.geom)
+        #     if isinstance(new_polygon, MultiPolygon):
+        #         return CompoundGeometry(
+        #             [Geometry(polygon, material) for polygon in new_polygon.geoms]
+        #         )
+        #     if isinstance(new_polygon, GeometryCollection):
+        #         return None
+        #     # Check to see if assigned_control_point is still valid
+        #     if self.assigned_control_point and new_polygon.contains(
+        #         self.assigned_control_point
+        #     ):
+        #         return Geometry(new_polygon, material, self.control_points[0])
+        #     return Geometry(new_polygon, material)
+        # except:
+        #     raise ValueError(
+        #         f"Cannot perform 'difference' on these two objects: {self} - {other}"
+        #     )
 
     def __add__(self, other):
         """
@@ -1318,10 +1369,19 @@ class CompoundGeometry(Geometry):
                 If true, all non parallel surfaces are filtered out.
                 Default is False.
         """
+        try:
+            import sectionproperties.pre.rhino as rhino_importer  # type: ignore
+        except ImportError as e:
+            print(e)
+            print(
+                "There is something wrong with your rhino library installation. "
+                "Please report this error at https://github.com/robbievanleeuwen/section-properties/issues"
+            )
+            return
         list_poly = rhino_importer.load_3dm(filepath, **kwargs)
         return cls(geoms=MultiPolygon(list_poly))
 
-    def create_mesh(self, mesh_sizes: List[float]):
+    def create_mesh(self, mesh_sizes: List[float], coarse: bool = False):
         """Creates a quadratic triangular mesh from the Geometry object.
 
         :param mesh_size: A float describing the maximum mesh element area to be
@@ -1329,6 +1389,8 @@ class CompoundGeometry(Geometry):
             CompoundGeometry object. If a list of length 1 is passed, then the one
             size will be applied to all constituent Geometry meshes.
         :type mesh_sizes: List[float]
+        :param bool coarse: If set to True, will create a coarse mesh (no area or
+            quality constraints)
 
         :return: Geometry-object with mesh data stored in .mesh attribute. Returned
             Geometry-object is self, not a new instance.
@@ -1353,7 +1415,12 @@ class CompoundGeometry(Geometry):
         if len(mesh_sizes) == 1:
             mesh_sizes = mesh_sizes * len(self.control_points)
         self.mesh = pre.create_mesh(
-            self.points, self.facets, self.holes, self.control_points, mesh_sizes
+            self.points,
+            self.facets,
+            self.holes,
+            self.control_points,
+            mesh_sizes,
+            coarse,
         )
         return self
 
@@ -1668,7 +1735,11 @@ class CompoundGeometry(Geometry):
                     )
 
         elif isinstance(unionized_poly, Polygon):
-            inadvertent_holes += Geometry(unionized_poly).holes
+            if Geometry(unionized_poly).holes:
+                inadvertent_holes += Geometry(unionized_poly).holes
+            else:  # Holes have been destroyed through operations
+                inadvertent_holes = []
+                self.holes = []
 
         extra_holes = []
         if set(inadvertent_holes) - set(self.holes):
@@ -1863,3 +1934,14 @@ def round_polygon_vertices(poly: Polygon, tol: int) -> Polygon:
     if not rounded_exterior.any():
         return Polygon()
     return Polygon(rounded_exterior, rounded_interiors)
+
+
+def check_geometry_overlaps(lop: List[Polygon]) -> bool:
+    """
+    Returns True if any of the Polygon in the list of Polygons, 'lop',
+    are overlapping with any other Polygons in 'lop'. Returns False
+    if all Polygon are disjoint.
+    """
+    union_area = unary_union(lop).area
+    sum_polygons = sum([poly.area for poly in lop])
+    return union_area != sum_polygons
